#ifndef MINIMISATION_HH
#define MINIMISATION_HH

#include <dune/common/fmatrix.hh>
#include <dune/common/fvector.hh>
#include <dune/common/stdstreams.hh>

#include <dune/fufem/arithmetic.hh>
#include <dune/fufem/interval.hh>
#include <dune/tnnmg/problem-classes/bisection.hh>

#include "mydirectionalconvexfunction.hh"

namespace Dune {
template <class Functional>
void descentMinimisation(Functional const &J,
                         typename Functional::SmallVector &x,
                         typename Functional::SmallVector const &v,
                         Bisection const &bisection) {
  using SmallVector = typename Functional::SmallVector;
  using LocalNonlinearityType = typename Functional::NonlinearityType;

  MyDirectionalConvexFunction<LocalNonlinearityType> const JRest(
      computeDirectionalA(J.A, v), computeDirectionalb(J.A, J.b, x, v), *J.phi,
      x, v);
  // }}}

  int count;
  double const stepsize = bisection.minimize(JRest, 0.0, 0.0, count);

  Arithmetic::addProduct(x, stepsize, v);
}

template <class Functional>
void minimise(Functional const &J, typename Functional::SmallVector &x,
              size_t steps, Bisection const &bisection) {
  using SmallVector = typename Functional::SmallVector;

  for (size_t step = 0; step < steps; ++step) {
    SmallVector v;
    J.gradient(x, v);
    v *= -1;

    if (v.two_norm() < 1e-14) // TODO: Make controllable
      return;

    descentMinimisation(J, x, v, bisection);
  }
}
}
#endif
