#ifndef RUINA_STATE_UPDATER_HH
#define RUINA_STATE_UPDATER_HH

#include "compute_state_ruina.hh"
#include "stateupdater.hh"

template <class SingletonVectorType, class VectorType>
class RuinaStateUpdater : public StateUpdater<SingletonVectorType, VectorType> {
public:
  RuinaStateUpdater(SingletonVectorType _alpha_initial,
                    Dune::BitSetVector<1> const &_nodes, double _L);

  virtual void nextTimeStep();
  virtual void setup(double _tau);
  virtual void solve(VectorType const &velocity_field);
  virtual void extractState(SingletonVectorType &state);

private:
  SingletonVectorType alpha_o;
  SingletonVectorType alpha;
  Dune::BitSetVector<1> const &nodes;
  double const L;
  double tau;
};

template <class SingletonVectorType, class VectorType>
RuinaStateUpdater<SingletonVectorType, VectorType>::RuinaStateUpdater(
    SingletonVectorType _alpha_initial, Dune::BitSetVector<1> const &_nodes,
    double _L)
    : alpha(_alpha_initial), nodes(_nodes), L(_L) {}

template <class SingletonVectorType, class VectorType>
void RuinaStateUpdater<SingletonVectorType, VectorType>::nextTimeStep() {
  alpha_o = alpha;
}

template <class SingletonVectorType, class VectorType>
void RuinaStateUpdater<SingletonVectorType, VectorType>::setup(double _tau) {
  tau = _tau;
}

template <class SingletonVectorType, class VectorType>
void RuinaStateUpdater<SingletonVectorType, VectorType>::solve(
    VectorType const &velocity_field) {
  for (size_t i = 0; i < nodes.size(); ++i)
    if (nodes[i][0]) {
      double const V = velocity_field[i].two_norm();
      alpha[i] = state_update_ruina(tau, V * tau / L, alpha_o[i]);
    }
}

template <class SingletonVectorType, class VectorType>
void RuinaStateUpdater<SingletonVectorType, VectorType>::extractState(
    SingletonVectorType &state) {
  state = alpha;
}

#endif
