#ifndef DUNE_TECTONIC_TIMESTEPPING_BACKWARD_EULER_HH
#define DUNE_TECTONIC_TIMESTEPPING_BACKWARD_EULER_HH

template <class VectorType, class MatrixType, class FunctionType, int dim>
class BackwardEuler
    : public TimeSteppingScheme<VectorType, MatrixType, FunctionType, dim> {
public:
  BackwardEuler(MatrixType const &_A, MatrixType const &_M,
                VectorType const &_u_initial, VectorType const &_v_initial,
                Dune::BitSetVector<dim> const &_dirichletNodes,
                FunctionType const &_dirichletFunction);

  void virtual nextTimeStep() override;
  void virtual setup(VectorType const &, double, double, VectorType &,
                     VectorType &, MatrixType &) override;
  void virtual postProcess(VectorType const &) override;
  void virtual extractDisplacement(VectorType &) const override;
  void virtual extractVelocity(VectorType &) const override;

private:
  MatrixType const &A;
  MatrixType const &M;
  VectorType u;
  VectorType v;
  Dune::BitSetVector<dim> const &dirichletNodes;
  FunctionType const &dirichletFunction;

  VectorType u_o;
  VectorType v_o;

  double tau;

  bool postProcessCalled = false;
};
#endif
